import RPi.GPIO as GPIO
import time

# Pin configuration
channel_a = 17  # GPIO pin for Channel A
channel_b = 27  # GPIO pin for Channel B

# Encoder specifications
PPR = 5  # Pulses per revolution
time_interval = 1.0  # Time interval for speed calculation in seconds

# Variables
pulse_count = 0
direction = None
last_time = time.time()

def setup():
    GPIO.setmode(GPIO.BCM)
    GPIO.setup(channel_a, GPIO.IN, pull_up_down=GPIO.PUD_UP)
    GPIO.setup(channel_b, GPIO.IN, pull_up_down=GPIO.PUD_UP)
    GPIO.add_event_detect(channel_a, GPIO.RISING, callback=encoder_callback)

def encoder_callback(channel):
    global pulse_count, direction
    channel_b_state = GPIO.input(channel_b)
    if channel_b_state == 0:
        direction = "Clockwise"
        pulse_count += 1
    else:
        direction = "Counterclockwise"
        pulse_count -= 1

def calculate_speed():
    global pulse_count, last_time
    current_time = time.time()
    elapsed_time = current_time - last_time
    last_time = current_time

    # Calculate revolutions
    revolutions = pulse_count / PPR
    pulse_count = 0  # Reset pulse count after calculation

    # Calculate RPM
    rpm = (revolutions / elapsed_time) * 60
    return rpm

def main():
    try:
        setup()
        print("Reading encoder data and calculating speed...")
        while True:
            time.sleep(time_interval)  # Wait for the interval
            speed = calculate_speed()
            print(f"Speed: {speed:.2f} RPM, Direction: {direction}")
    except KeyboardInterrupt:
        print("\nExiting...")
        GPIO.cleanup()

if __name__ == "__main__":
    main()
