from gpiozero import PWMOutputDevice, RotaryEncoder
from time import sleep, time

class Motor():
    def __init__(self, EnaA, In1A, In2A, EnaB, In1B, In2B, encA_pins=None, encB_pins=None):
        # Initialize motor control pins
        self.pwmA = PWMOutputDevice(EnaA)
        self.in1A = PWMOutputDevice(In1A)
        self.in2A = PWMOutputDevice(In2A)
        self.pwmB = PWMOutputDevice(EnaB)
        self.in1B = PWMOutputDevice(In1B)
        self.in2B = PWMOutputDevice(In2B)
        
        # Initialize encoders if pins are provided
        if encA_pins:
            self.encoderA = RotaryEncoder(a=encA_pins[0], b=encA_pins[1], max_steps=0)
            self.encoderA.steps = 0  # Reset steps
            self.encoderA.when_rotated = self.on_rotate_left
        else:
             self.encoderA = None
            
        if encB_pins:
            self.encoderB = RotaryEncoder(a=encB_pins[0], b=encB_pins[1], max_steps=0)
            self.encoderB.steps = 0  # Reset steps
            self.encoderB.when_rotated = self.on_rotate_right
        else:
            self.encoderB = None
    
    # Callback for left encoder rotation
    def on_rotate_left(self):
        steps = self.encoderA.steps
        print(f"Left Encoder Steps: {steps} trigger")
    
    # Callback for right encoder rotation
    def on_rotate_right(self):
        steps = self.encoderB.steps
        #print(f"Right Encoder Steps: {steps}")
        
    def move(self, speed=0.5, turn=0, t=0):
        # Ensure speed and turn are within bounds
        speed = max(-1, min(1, speed))  # Clamp speed to [-1, 1]
        turn = max(-1, min(1, turn))    # Clamp turn to [-1, 1]
        
        # Calculate speed for each motor
        leftSpeed = speed - turn
        rightSpeed = speed + turn
        
        # Clamp motor speeds to [-1, 1]
        leftSpeed = max(-1, min(1, leftSpeed))
        rightSpeed = max(-1, min(1, rightSpeed))
        
        # Set left motor
        self.pwmA.value = abs(leftSpeed)
        self.in1A.value = leftSpeed > 0
        self.in2A.value = leftSpeed <= 0
        
        # Set right motor
        self.pwmB.value = abs(rightSpeed)
        self.in1B.value = rightSpeed > 0
        self.in2B.value = rightSpeed <= 0
        
        # Print debugging info
        print(f"Left Motor: Speed={leftSpeed}, Right Motor: Speed={rightSpeed}")
        
        # Run the motors for time 't' while monitoring encoder steps
        start_time = time()
        while time() - start_time < t:
            # Optionally, read and print encoder values
            if self.encoderA:
                left_steps = self.encoderA.steps
                print(f"Left Encoder Steps: {left_steps}")
            if self.encoderB:
                right_steps = self.encoderB.steps
#                 print(f"Right Encoder Steps: {right_steps}")
            sleep(0.1)  # Adjust sleep time as needed
            
    def stop(self, t=0):
        self.pwmA.value = 0
        self.pwmB.value = 0
        sleep(t)

def main():
    # Define encoder GPIO pins for left and right encoders
    # Replace these with your actual GPIO pins
    left_encoder_pins = (5, 6)     # GPIO pins for left encoder channels A and B
    right_encoder_pins = (13, 19)  # GPIO pins for right encoder channels A and B
    
    # Initialize motor with encoder pins
    motor1 = Motor(2, 3, 4, 17, 22, 27, encA_pins=left_encoder_pins, encB_pins=right_encoder_pins)
    
    # Run motor commands
    motor1.move(0.5, 0, 10)  # Move forward for 3 seconds
    motor1.stop(1)
    '''
    motor1.move(-0.5, 0, 3)  # Move backward for 3 seconds
    motor1.stop(1)
    motor1.move(0.5, 0.5, 3)  # Turn right for 3 seconds
    motor1.stop(1)
    motor1.move(0.5, -0.5, 3)  # Turn left for 3 seconds
    motor1.stop(1)
    '''

if __name__ == '__main__':
    main()
