from gpiozero import RotaryEncoder, PWMOutputDevice
from time import sleep

class Motor():
    def __init__(self, EnaA, In1A, In2A, EnaB, In1B, In2B):
        self.pwmA = PWMOutputDevice(EnaA)
        self.in1A = PWMOutputDevice(In1A)
        self.in2A = PWMOutputDevice(In2A)
        self.pwmB = PWMOutputDevice(EnaB)
        self.in1B = PWMOutputDevice(In1B)
        self.in2B = PWMOutputDevice(In2B)
        
    def move(self, speed=0.5, turn=0, t=0):
        # Ensure speed and turn are within bounds
        speed = max(-1, min(1, speed))  # Clamp speed to [-1, 1]
        turn = max(-1, min(1, turn))    # Clamp turn to [-1, 1]
        
        # Calculate speed for each motor
        leftSpeed = speed - turn
        rightSpeed = speed + turn
        
        # Clamp motor speeds to [-1, 1]
        leftSpeed = max(-1, min(1, leftSpeed))
        rightSpeed = max(-1, min(1, rightSpeed))
        
        # Set left motor
        self.pwmA.value = abs(leftSpeed)
        self.in1A.value = leftSpeed > 0
        self.in2A.value = leftSpeed <= 0
        
        # Set right motor
        self.pwmB.value = abs(rightSpeed)
        self.in1B.value = rightSpeed > 0
        self.in2B.value = rightSpeed <= 0
        
        # Print debugging info
        print(f"Left Motor: Speed={leftSpeed}, Right Motor: Speed={rightSpeed}")
        
        sleep(t)
        
    def stop(self, t=0):
        self.pwmA.value = 0
        self.pwmB.value = 0
        sleep(t)


# Encoder GPIO pins (updated to avoid conflicts)
MOTOR1_HALL_A = 5
MOTOR1_HALL_B = 6
MOTOR2_HALL_A = 13
MOTOR2_HALL_B = 19

# Create RotaryEncoder objects for encoders
motor1_encoder = RotaryEncoder(MOTOR1_HALL_A, MOTOR1_HALL_B, max_steps=0)
motor2_encoder = RotaryEncoder(MOTOR2_HALL_A, MOTOR2_HALL_B, max_steps=0)

# Callback functions for rotation events
def motor1_rotated():
    print(f"Motor 1: Steps={motor1_encoder.steps}, Direction={'Clockwise' if motor1_encoder.value > 0 else 'Counter-Clockwise'}")

def motor2_rotated():
    print(f"Motor 2: Steps={motor2_encoder.steps}, Direction={'Clockwise' if motor2_encoder.value > 0 else 'Counter-Clockwise'}")

# Attach callbacks to each encoder
motor1_encoder.when_rotated = motor1_rotated
motor2_encoder.when_rotated = motor2_rotated

def main():
    print("Starting motor and monitoring encoder outputs...")
    motor1.move(0.1, 0, 3)  # Move forward for 3 seconds
    motor1.stop(1)
    # You can add more motor movements here
    print("Test complete.")
    # Keep the program running to continue monitoring encoders
    while True:
        sleep(1)

if __name__ == '__main__':
    motor1 = Motor(2, 3, 4, 17, 22, 27)
    try:
        main()
    except KeyboardInterrupt:
        print("\nExiting program...")
