import smbus
import time

class BNO055:
    def __init__(self, i2c_bus=1, address=0x28):
        self.bus = smbus.SMBus(i2c_bus)
        self.address = address
        self.init_bno055()

    def write_register(self, register, value):
        self.bus.write_byte_data(self.address, register, value)

    def read_register(self, register, length=1):
        if length == 1:
            return self.bus.read_byte_data(self.address, register)
        else:
            return self.bus.read_i2c_block_data(self.address, register, length)

    def init_bno055(self):
        # Switch to CONFIG mode to configure sensor
        self.write_register(0x3D, 0x00)
        time.sleep(0.05)

        # Set power mode to Normal
        self.write_register(0x3E, 0x00)

        # Set to NDOF mode (Full sensor fusion for orientation)
        self.write_register(0x3D, 0x0C)
        time.sleep(0.05)

    def read_calibration_status(self):
        # Read the calibration status byte (CALIB_STAT register: 0x35)
        calib = self.read_register(0x35)
        sys = (calib >> 6) & 0x03  # System calibration status (bits 6-7)
        gyro = (calib >> 4) & 0x03  # Gyroscope calibration status (bits 4-5)
        accel = (calib >> 2) & 0x03  # Accelerometer calibration status (bits 2-3)
        mag = calib & 0x03          # Magnetometer calibration status (bits 0-1)
        return sys, gyro, accel, mag

if __name__ == "__main__":
    sensor = BNO055()
    print("Starting BNO055 calibration process...")
    
    while True:
        sys, gyro, accel, mag = sensor.read_calibration_status()
        print(f"System: {sys}, Gyro: {gyro}, Accel: {accel}, Mag: {mag}")
        
        if sys == 3 and gyro == 3 and accel == 3 and mag == 3:
            print("Calibration Complete!")
            break
        
        print("Follow these steps to calibrate:")
        print("- Keep sensor still to calibrate gyroscope.")
        print("- Move sensor in all axes to calibrate accelerometer.")
        print("- Perform figure-8 motion for magnetometer.")
        
        time.sleep(1)
'''
def read_calibration_data(self):
    return self.read_register(0x55, 22)  # 22 bytes of calibration data

def write_calibration_data(self, data):
    for i in range(22):
        self.write_register(0x55 + i, data[i])
'''