from gpiozero import PWMOutputDevice, Button
from time import sleep

class Motor():
    def __init__(self, EnaA, In1A, In2A, EnaB, In1B, In2B, HallA_A, HallA_B, HallB_A, HallB_B):
        # Motor control initialization
        self.pwmA = PWMOutputDevice(EnaA)
        self.in1A = PWMOutputDevice(In1A)
        self.in2A = PWMOutputDevice(In2A)
        self.pwmB = PWMOutputDevice(EnaB)
        self.in1B = PWMOutputDevice(In1B)
        self.in2B = PWMOutputDevice(In2B)
        
        # Encoder initialization
        self.motor1_hall_a = Button(HallA_A, pull_up=True)
        self.motor1_hall_b = Button(HallA_B, pull_up=True)
        self.motor2_hall_a = Button(HallB_A, pull_up=True)
        self.motor2_hall_b = Button(HallB_B, pull_up=True)
        
        # Attach callbacks
        self.motor1_hall_a.when_pressed = self.motor1_channel_a_detected
        self.motor1_hall_b.when_pressed = self.motor1_channel_b_detected
        self.motor2_hall_a.when_pressed = self.motor2_channel_a_detected
        self.motor2_hall_b.when_pressed = self.motor2_channel_b_detected

        # Encoder counters
        self.motor1_count = 0
        self.motor2_count = 0
        
    def move(self, speed=0.5, turn=0, t=0):
        # Ensure speed and turn are within bounds
        speed = max(-1, min(1, speed))  # Clamp speed to [-1, 1]
        turn = max(-1, min(1, turn))    # Clamp turn to [-1, 1]
        
        # Calculate speed for each motor
        leftSpeed = speed - turn
        rightSpeed = speed + turn
        
        # Clamp motor speeds to [-1, 1]
        leftSpeed = max(-1, min(1, leftSpeed))
        rightSpeed = max(-1, min(1, rightSpeed))
        
        # Set left motor
        self.pwmA.value = abs(leftSpeed)
        self.in1A.value = leftSpeed > 0
        self.in2A.value = leftSpeed <= 0
        
        # Set right motor
        self.pwmB.value = abs(rightSpeed)
        self.in1B.value = rightSpeed > 0
        self.in2B.value = rightSpeed <= 0
        
        # Print debugging info
        print(f"Left Motor: Speed={leftSpeed}, Right Motor: Speed={rightSpeed}")
        sleep(t)
        

            
    def stop(self, t=0):
        self.pwmA.value = 0
        self.pwmB.value = 0
        sleep(t)




    # Encoder callback functions
    def motor1_channel_a_detected(self):
        self.motor1_count += 1
        print(f"Motor 1: Channel A pulse detected, Count: {self.motor1_count}")

    def motor1_channel_b_detected(self):
        print(f"Motor 1: Channel B pulse detected")


    def motor2_channel_a_detected(self):
        self.motor2_count += 1
        print(f"Motor 2: Channel A pulse detected, Count: {self.motor2_count}")

    def motor2_channel_b_detected(self):
        print(f"Motor 2: Channel B pulse detected")
        


    
def main():
    
    try:
        print("Starting motor movement and encoder monitoring. Press Ctrl+C to exit.")
        
        motor1.move(0.1, 0, 10)  # Move forward
        motor1.stop(1)



    except KeyboardInterrupt:
        print("\nExiting program...")
        


if __name__ == '__main__':
    motor1 = Motor(2, 3, 4, 17, 22, 27, 5,6,20,21 )
    main()

